import { Test, TestingModule } from '@nestjs/testing';
import { CreateAdService } from '@modules/ad/core/commands/create-ad/create-ad.service';
import { AD_REPOSITORY, PARAMS_PROVIDER } from '@modules/ad/ad.di-tokens';
import { DefaultParamsProviderPort } from '@modules/ad/core/ports/default-params-provider.port';
import { WaypointDTO } from '@modules/ad/interface/grpc-controllers/dtos/waypoint.dto';
import { CreateAdRequestDTO } from '@modules/ad/interface/grpc-controllers/dtos/create-ad.request.dto';
import { Frequency } from '@modules/ad/core/ad.types';
import { CreateAdCommand } from '@modules/ad/core/commands/create-ad/create-ad.command';
import { Result } from 'oxide.ts';
import { AggregateID } from '@libs/ddd';
import { AdAlreadyExistsError } from '@modules/ad/core/ad.errors';
import { AdEntity } from '@modules/ad/core/ad.entity';

const originWaypoint: WaypointDTO = {
  position: 0,
  lon: 48.68944505415954,
  lat: 6.176510296462267,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: WaypointDTO = {
  position: 1,
  lon: 48.8566,
  lat: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};
const punctualCreateAdRequest: CreateAdRequestDTO = {
  userId: '4eb6a6af-ecfd-41c3-9118-473a507014d4',
  fromDate: '2023-12-21',
  toDate: '2023-12-21',
  schedule: {
    thu: '08:15',
  },
  driver: true,
  passenger: true,
  seatsRequested: 1,
  frequency: Frequency.PUNCTUAL,
  waypoints: [originWaypoint, destinationWaypoint],
};

const mockAdRepository = {
  insert: jest.fn(),
};

const mockDefaultParamsProvider: DefaultParamsProviderPort = {
  getParams: () => {
    return {
      MON_MARGIN: 900,
      TUE_MARGIN: 900,
      WED_MARGIN: 900,
      THU_MARGIN: 900,
      FRI_MARGIN: 900,
      SAT_MARGIN: 900,
      SUN_MARGIN: 900,
      DRIVER: false,
      SEATS_PROPOSED: 3,
      PASSENGER: true,
      SEATS_REQUESTED: 1,
      STRICT: false,
      DEFAULT_TIMEZONE: 'Europe/Paris',
    };
  },
};

describe('create-ad.service', () => {
  let createAdService: CreateAdService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        {
          provide: PARAMS_PROVIDER,
          useValue: mockDefaultParamsProvider,
        },
        CreateAdService,
      ],
    }).compile();

    createAdService = module.get<CreateAdService>(CreateAdService);
  });

  it('should be defined', () => {
    expect(createAdService).toBeDefined();
  });

  describe('execution', () => {
    const createAdCommand = new CreateAdCommand(punctualCreateAdRequest);
    it('should create a new ad', async () => {
      AdEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      const result: Result<AggregateID, AdAlreadyExistsError> =
        await createAdService.execute(createAdCommand);
      expect(result.unwrap()).toBe('047a6ecf-23d4-4d3e-877c-3225d560a8da');
    });
  });
});
