import { Controller, Get } from '@nestjs/common';
import { HealthCheckService, HealthCheck } from '@nestjs/terminus';
import { RepositoriesHealthIndicatorUseCase } from '../../core/usecases/repositories.health-indicator.usecase';

@Controller('health')
export class HealthHttpController {
  constructor(
    private readonly repositoriesHealthIndicatorUseCase: RepositoriesHealthIndicatorUseCase,
    private healthCheckService: HealthCheckService,
  ) {}

  @Get()
  @HealthCheck()
  async check() {
    try {
      return await this.healthCheckService.check([
        async () =>
          this.repositoriesHealthIndicatorUseCase.isHealthy('repositories'),
      ]);
    } catch (error) {
      throw error;
    }
  }
}
